@@ -13,7 +13,7 @@ from ffta.pixel_utils import badpixels
 import os
 import numpy as np
 from ffta.load import get_utils
-from pyUSID.processing.comp_utils import parallel_compute
+from sidpy.proc.comp_utils import parallel_compute
 from pyUSID.io.write_utils import Dimension
 import h5py
 
@@ -22,7 +22,8 @@ from scipy.ndimage import gaussian_filter1d
 from ffta.hdf_utils.process import FFtrEFM
 
 from matplotlib import pyplot as plt
-
+from numpy.linalg import LinAlgError
+import warnings
 '''
 To do:
     Separate the instantaneous frequency and tFP/shift calculations
@@ -50,48 +51,59 @@ class GKPFM(FFtrEFM):
         >> data._get_existing_datasets()
     """
 
-    def __init__(self, h5_main, parm_dict = {}, can_params = {}, 
-                 pixel_params ={}, TF_norm=[], exc_wfm=[], periods = 2, 
-                 tip_response = '', tip_excitation = '', exc_wfm_file = '',
-                 override=False, noise_tolerance = 1e-4, **kwargs):
+    def __init__(self, h5_main, parm_dict={}, can_params={},
+                 pixel_params={}, TF_norm=[], exc_wfm=[], periods=2,
+                 tip_response='', tip_excitation='', exc_wfm_file='',
+                 override=False, exc_floor=10, noise_tolerance=1e-4, **kwargs):
         """
         Parameters
         ----------
         h5_main : h5py.Dataset object
             Dataset to process
-
         parm_dict : dict, optional
             Additional updates to the parameters dictionary. e.g. changing the trigger.
             You can also explicitly update self.parm_dict.update({'key': value})
-        
+            The default is {}.
         can_params : dict, optional
             Cantilever parameters describing the behavior
             Can be loaded from ffta.pixel_utils.load.cantilever_params
-        
+            The default is {}.
         pixel_params : dict, optional
-
-    
+            Currently unimplemented. The default is {}.
+        TF_norm : numpy array, optional
+            Transfer function (normalized). Can use GKPixel to generate, or supply explicitly
+        exc_wfm : numpy array, optional
+            The excitation used to generated Transfer Function. Unneeded if supplying a TF explicitly
+        periods : int, optional
+            Num of cantilever cycles to process in CPD calc. The default is 2.
+        tip_response : string, optional
+            File path to tip_response .ibw. Only needed if calculating TF_norm here
+        tip_excitation : string, optional
+            File path to tip excitation .ibw. Only needed if calculating TF_norm here
+        exc_wfm_file : string, optional
+            File path to .ibw for excitation during actual measurement (not TF).
         override : bool, optional
             If True, forces creation of new results group. Use in _get_existing_datasets
-    
-        kwargs : dictionary or variable
-            Keyword pairs to pass to Process constructor
+        exc_floor : float, optional
+            The noise floor for calculating Transfer Function. The default is 10.
+        noise_tolerance : float, optional
+            Use to determine noise_floor in reconstructing force. The default is 1e-4.
         """
 
         self.parm_dict = parm_dict
-        
+
         for key, val in parm_dict.items():
             self.parm_dict.update({key: val})
-                
+
         if any(can_params):
-            if 'Initial' in can_params: #only care about the initial conditions
+            if 'Initial' in can_params:  # only care about the initial conditions
                 for key, val in can_params['Initial'].items():
                     self.parm_dict.update({key: val})
             else:
                 for key, val in can_params.items():
                     self.parm_dict.update({key: val})
-                    
-        self.pixel_params = pixel_params    
+
+        self.pixel_params = pixel_params
         self.override = override
         self.parm_dict['tip_response'] = tip_response
         self.parm_dict['tip_excitation'] = tip_excitation
@@ -99,23 +111,24 @@ class GKPFM(FFtrEFM):
 
         self.parm_dict['periods'] = periods
         self.parm_dict['noise_tolerance'] = noise_tolerance
-           
-        super().__init__(h5_main, parm_dict=self.parm_dict ,
-                                    can_params = can_params, process_name='GKPFM', 
-                                    **kwargs)
+
+        super().__init__(h5_main, parm_dict=self.parm_dict,
+                         can_params=can_params, process_name='GKPFM',
+                         **kwargs)
 
         # save Transfer Function
-        defl = get_utils.get_pixel(self.h5_main, [0,0], array_form=True).flatten()
-        _gk = GKPixel(defl, self.parm_dict, exc_wfm = exc_wfm)
-        _gk.load_tf(self.parm_dict['tip_response'], self.parm_dict['tip_excitation'])
-        _gk.process_tf()
-        
-        self.TF_norm = _gk.TF_norm
-        del _gk
-        
+        self.process_tf(exc_floor)
+        # defl = get_utils.get_pixel(self.h5_main, [0, 0], array_form=True).flatten()
+        # _gk = GKPixel(defl, self.parm_dict, exc_wfm=exc_wfm)
+        # _gk.load_tf(self.parm_dict['tip_response'], self.parm_dict['tip_excitation'])
+        # _gk.process_tf()
+
+        # self.TF_norm = _gk.TF_norm
+        # del _gk
+
         self.parm_dict['denoise'] = False
         self.parm_dict['filter_cpd'] = False
-        
+
         return
 
     def update_parm(self, **kwargs):
@@ -126,8 +139,31 @@ class GKPFM(FFtrEFM):
         self.parm_dict.update(kwargs)
 
         return
+    
+    def process_tf(self, exc_floor=10):
+        """
+        Generates a Transfer Function using GKPixel function
+        
+        Parameters
+        ----------
+        exc_floor : float
+            DESCRIPTION.
+        """
+        
+        defl = get_utils.get_pixel(self.h5_main, [0, 0], array_form=True).flatten()
+        _gk = GKPixel(defl, self.parm_dict)
+        _gk.load_tf(self.parm_dict['tip_response'], self.parm_dict['tip_excitation'])
+        _gk.process_tf(exc_floor=exc_floor)
 
-    def test(self, pixel_ind=[0,0]):
+        self.TF_norm = _gk.TF_norm
+        del _gk
+        
+        if any(np.isnan(self.TF_norm)):
+            warnings.warn('TF is NaN! Lower exc_floor value to lower threshold of pass TF_norm explicitly')
+        
+        return
+
+    def test(self, pixel_ind=[0, 0], phases_to_test=[2.0708, 2.1208, 2.1708]):
         """
         Test the Pixel analysis of a single pixel
 
@@ -136,7 +172,10 @@ class GKPFM(FFtrEFM):
         pixel_ind : uint or list
             Index of the pixel in the dataset that the process needs to be tested on.
             If a list it is read as [row, column]
-
+        phases_to_test : list, optional
+            Which phases to shift the signal with. The default is [2.0708, 2.1208, 2.1708],
+            which is 0.5, 0.55, 0.5 + pi/2
+            
         Returns
         -------
         [inst_freq, tfp, shift] : List
@@ -158,34 +197,46 @@ class GKPFM(FFtrEFM):
         # as an array, not an ffta.Pixel
         defl = get_utils.get_pixel(self.h5_main, pixel_ind, array_form=True).flatten()
 
-        _gk = GKPixel(defl, self.parm_dict, exc_wfm = self.exc_wfm, 
-                      TF_norm = self.TF_norm)        
-        _gk.force_out(plot=True, noise_tolerance=self.parm_dict['noise_tolerance'])
-        _gk.min_phase()
+        _gk = GKPixel(defl, self.parm_dict, exc_wfm=self.exc_wfm,
+                      TF_norm=self.TF_norm)
+
+        _gk.min_phase(phases_to_test=phases_to_test, noise_tolerance=self.parm_dict['noise_tolerance'])
         
+        # If User supplies a phase shift
+        if 'phase_shift' in self.parm_dict:
+            _gk.force_out(plot=True, noise_tolerance=self.parm_dict['noise_tolerance'],
+                          phase_shift = self.parm_dict['phase_shift'])
+        else:
+            _gk.force_out(plot=True, noise_tolerance=self.parm_dict['noise_tolerance'])
+
         if self.parm_dict['denoise']:
             print('aa')
             _gk.noise_filter()
-        
-        _gk.analyze_cpd(use_raw=False, periods=self.parm_dict['periods'])
-        
+
+        try:
+            _gk.analyze_cpd(use_raw=False, periods=self.parm_dict['periods'])
+        except LinAlgError:
+            print('Make sure TF_norm is set correctly. Pass TF_norm explicitly.')
+            return _gk
+
         if self.parm_dict['filter_cpd']:
             print('bb')
             _gk.CPD = gaussian_filter1d(_gk.CPD, 1)[:_gk.num_CPD]
-        
-        plt.figure()
-        plt.plot(_gk.CPD)
-        
+
+        _gk.plot_cpd()
+
         self.cpd_dict = _gk._calc_cpd_params(return_dict=True, periods=self.parm_dict['periods'])
 
-        _,_,_, = self._map_function(defl, self.parm_dict, self.TF_norm, self.exc_wfm)
-    
-        return
+        _, _, _, = self._map_function(defl, self.parm_dict, self.TF_norm, self.exc_wfm)
+
+        return _gk
 
     def _create_results_datasets(self):
         '''
         Creates the datasets an Groups necessary to store the results.
 
+        Parameters
+        ----------
         h5_if : 'Inst_Freq' h5 Dataset
             Contains the Instantaneous Frequencies
             
@@ -196,7 +247,7 @@ class GKPFM(FFtrEFM):
             Contains the frequency shift data as a 1D matrix
         '''
 
-        print ('Creating results datasets')
+        print('Creating CPD results datasets')
 
         # Get relevant parameters
         num_rows = self.parm_dict['num_rows']
@@ -207,7 +258,7 @@ class GKPFM(FFtrEFM):
         cpd_ds_shape = [num_rows * num_cols, self.cpd_dict['num_CPD']]
 
         self.h5_results_grp = usid.hdf_utils.create_results_group(self.h5_main, self.process_name)
-        self.h5_cpd_grp = usid.hdf_utils.create_results_group(self.h5_main, self.process_name+'_CPD')
+        self.h5_cpd_grp = usid.hdf_utils.create_results_group(self.h5_main, self.process_name + '_CPD')
 
         usid.hdf_utils.copy_attributes(self.h5_main.parent, self.h5_results_grp)
         usid.hdf_utils.copy_attributes(self.h5_main.parent, self.h5_cpd_grp)
@@ -231,35 +282,33 @@ class GKPFM(FFtrEFM):
                                                           spec_desc,  # Spectroscopic dimensions
                                                           dtype=np.float32,  # data type / precision
                                                           main_dset_attrs=self.parm_dict)
-        
 
         self.h5_cpd = usid.hdf_utils.write_main_dataset(self.h5_cpd_grp,
-                                                       cpd_ds_shape,
-                                                       'CPD',  # Name of main dataset
-                                                       'Potential',  # Physical quantity contained in Main dataset
-                                                       'V',  # Units for the physical quantity
-                                                       None,  # Position dimensions
-                                                       cpd_spec_desc,  # Spectroscopic dimensions
-                                                       h5_pos_inds = self.h5_main.h5_pos_inds, # Copy Pos Dimensions
-                                                       h5_pos_vals = self.h5_main.h5_pos_vals, 
-                                                       dtype=np.float32,  # data type / precision
-                                                       main_dset_attrs=self.parm_dict)
-        
+                                                        cpd_ds_shape,
+                                                        'CPD',  # Name of main dataset
+                                                        'Potential',  # Physical quantity contained in Main dataset
+                                                        'V',  # Units for the physical quantity
+                                                        None,  # Position dimensions
+                                                        cpd_spec_desc,  # Spectroscopic dimensions
+                                                        h5_pos_inds=self.h5_main.h5_pos_inds,  # Copy Pos Dimensions
+                                                        h5_pos_vals=self.h5_main.h5_pos_vals,
+                                                        dtype=np.float32,  # data type / precision
+                                                        main_dset_attrs=self.parm_dict)
 
         self.h5_cap = usid.hdf_utils.write_main_dataset(self.h5_cpd_grp,
-                                                       cpd_ds_shape,
-                                                       'capacitance',  # Name of main dataset
-                                                       'Capacitance',  # Physical quantity contained in Main dataset
-                                                       'F',  # Units for the physical quantity
-                                                       None,  # Position dimensions
-                                                       None,
-                                                       h5_pos_inds = self.h5_main.h5_pos_inds, # Copy Pos Dimensions
-                                                       h5_pos_vals = self.h5_main.h5_pos_vals, 
-                                                       h5_spec_inds = self.h5_cpd.h5_spec_inds, # Copy Spectroscopy Dimensions
-                                                       h5_spec_vals = self.h5_cpd.h5_spec_vals,
-                                                       dtype=np.float32,  # data type / precision
-                                                       main_dset_attrs=self.parm_dict)
-        
+                                                        cpd_ds_shape,
+                                                        'capacitance',  # Name of main dataset
+                                                        'Capacitance',  # Physical quantity contained in Main dataset
+                                                        'F',  # Units for the physical quantity
+                                                        None,  # Position dimensions
+                                                        None,
+                                                        h5_pos_inds=self.h5_main.h5_pos_inds,  # Copy Pos Dimensions
+                                                        h5_pos_vals=self.h5_main.h5_pos_vals,
+                                                        h5_spec_inds=self.h5_cpd.h5_spec_inds,
+                                                        # Copy Spectroscopy Dimensions
+                                                        h5_spec_vals=self.h5_cpd.h5_spec_vals,
+                                                        dtype=np.float32,  # data type / precision
+                                                        main_dset_attrs=self.parm_dict)
 
         self.h5_cpd.file.flush()
 
@@ -302,8 +351,8 @@ class GKPFM(FFtrEFM):
 
         # write the results to the file
         self.h5_force[pos_in_batch, :] = _force
-        self.h5_cpd[pos_in_batch, :] = _cpd[:,:self.h5_cpd.shape[1]]
-        self.h5_cap[pos_in_batch, :] = _capacitance[:,:self.h5_cap.shape[1]]
+        self.h5_cpd[pos_in_batch, :] = _cpd[:, :self.h5_cpd.shape[1]]
+        self.h5_cap[pos_in_batch, :] = _capacitance[:, :self.h5_cap.shape[1]]
 
         return
 
@@ -314,13 +363,13 @@ class GKPFM(FFtrEFM):
         index = which existing dataset to get
         
         """
-        
-        if not self.override:
 
+        if not self.override:
             self.h5_results_grp = usid.hdf_utils.find_dataset(self.h5_main.parent, 'CPD')[index].parent
             self.h5_new_spec_vals = self.h5_results_grp['Spectroscopic_Values']
             self.h5_cpd = self.h5_results_grp['CPD']
             self.h5_capacitance = self.h5_results_grp['capacitance']
+            self.h5_force = usid.hdf_utils.find_dataset(self.h5_main.parent, 'CPD')[index].parent['force']
 
         return
 
@@ -347,7 +396,7 @@ class GKPFM(FFtrEFM):
         TF_norm = args[1]
         exc_wfm = args[2]
 
-        gk = GKPixel(defl, parm_dict, exc_wfm=exc_wfm, TF_norm=TF_norm)    
+        gk = GKPixel(defl, parm_dict, exc_wfm=exc_wfm, TF_norm=TF_norm)
         gk.force_out(noise_tolerance=parm_dict['noise_tolerance'])
 
         if parm_dict['denoise']:
@@ -361,7 +410,7 @@ class GKPFM(FFtrEFM):
 
         capacitance = gk.capacitance
         force = gk.force
-    
+
         return [force, cpd, capacitance]
 
 
@@ -369,6 +418,8 @@ def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False):
     """
     Saves the tfp, shift, and fixed_tfp as CSV files
     
+    Parameters
+    ----------
     h5_file : H5Py file of FFtrEFM class
         Reminder you can always type: h5_svd.file or h5_avg.file for this
     
@@ -378,18 +429,18 @@ def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False):
     append : str, optional
         text to append to file name
     """
-    
+
     h5_ff = h5_file
-    
+
     if isinstance(h5_file, ffta.hdf_utils.process.FFtrEFM):
         print('Saving from FFtrEFM Class')
         h5_ff = h5_file.h5_main.file
         h5_path = h5_file.h5_results_grp.name
-    
+
     elif not isinstance(h5_file, h5py.File):
         print('Saving from pyUSID object')
         h5_ff = h5_file.file
-    
+
     tfp = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'tfp')[0][()]
     # tfp_fixed = usid.hdf_utils.find_dataset(h5_file[h5_path], 'tfp_fixed')[0][()]
     shift = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'shift')[0][()]
@@ -415,7 +466,7 @@ def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False):
     return
 
 
-def plot_tfp(ffprocess, scale_tfp = 1e6, scale_shift=1, threshold = 2, **kwargs):
+def plot_tfp(ffprocess, scale_tfp=1e6, scale_shift=1, threshold=2, **kwargs):
     '''
     Quickly plots the tfp and shift data. If there's a height image in the h5_file associated
      with ffprocess, will plot that as well
@@ -436,8 +487,8 @@ def plot_tfp(ffprocess, scale_tfp = 1e6, scale_shift=1, threshold = 2, **kwargs)
     img_length = ffprocess.parm_dict['FastScanSize']
     img_height = ffprocess.parm_dict['SlowScanSize']
     kwarg = {'origin': 'lower', 'x_vec': img_length * 1e6,
-              'y_vec': img_height * 1e6, 'num_ticks': 5, 'stdevs': 3, 'show_cbar':True}
-    
+             'y_vec': img_height * 1e6, 'num_ticks': 5, 'stdevs': 3, 'show_cbar': True}
+
     for k, v in kwarg.items():
         if k not in kwargs:
             kwargs.update({k: v})
@@ -455,7 +506,7 @@ def plot_tfp(ffprocess, scale_tfp = 1e6, scale_shift=1, threshold = 2, **kwargs)
 
     tfp_ax.set_title('tFP Image')
     shift_ax.set_title('Shift Image')
-    
+
     tfp_fixed, _ = badpixels.fix_array(ffprocess.h5_tfp[()], threshold=threshold)
 
     tfp_image, cbar_tfp = usid.viz.plot_utils.plot_map(tfp_ax, tfp_fixed * scale_tfp,
@@ -463,7 +514,6 @@ def plot_tfp(ffprocess, scale_tfp = 1e6, scale_shift=1, threshold = 2, **kwargs)
     shift_image, cbar_sh = usid.viz.plot_utils.plot_map(shift_ax, ffprocess.h5_shift[()] * scale_shift,
                                                         cmap='inferno', **kwargs)
 
-    
     cbar_tfp.set_label('Time (us)', rotation=270, labelpad=16)
     cbar_sh.set_label('Frequency Shift (Hz)', rotation=270, labelpad=16)
     text = tfp_ax.text(num_cols / 2, num_rows + 3, '')
