@@ -6,7 +6,6 @@ __maintainer__ = "Rajiv Giridharagopal"
 __email__ = "rgiri@uw.edu"
 __status__ = "Development"
 
-import logging
 import numpy as np
 from scipy import signal as sps
 from scipy import optimize as spo
@@ -15,16 +14,14 @@ from scipy import integrate as spg
 
 from ffta.pixel_utils import noise
 from ffta.pixel_utils import parab
-from ffta.pixel_utils import fitting
 from ffta.pixel_utils import dwavelet
 from ffta.pixel_utils import tfp_calc
 
 from matplotlib import pyplot as plt
-
 import pywt
 
 import time
-
+import warnings
 
 class Pixel:
     """
@@ -33,21 +30,22 @@ class Pixel:
     Extracts Time-to-First-Peak (tFP) from digitized Fast-Free Time-Resolved
     Electrostatic Force Microscopy (FF-trEFM) signals [1-2]. It includes a few
     types of frequency analysis:
-        a) Hilbert Transform
-        b) Wavelet Transform
-        c) Hilbert-Huang Transform (EMD)
 
-    ----------
+    a) Hilbert Transform
+    b) Wavelet Transform
+    c) Short-Time Fourier Transform (STFT))
+
     signal_array : (n_points, n_signals) array_like
         2D real-valued signal array, corresponds to a pixel.
     params : dict
         Includes parameters for processing. The list of parameters is:
 
-        trigger = float (in seconds)
-        total_time = float (in seconds)
-        sampling_rate = int (in Hz)
+        trigger = float (in seconds) (required)
+        total_time = float (in seconds) (either this or sampling rate required)
+        sampling_rate = int (in Hz) (see above)
+        
+        These are often supplied but can be a default:        
         drive_freq = float (in Hz)
-
         roi = float (in seconds)
         window = string (see documentation of scipy.signal.get_window)
         bandpass_filter = int (0: no filtering, 1: FIR filter, 2: IIR filter)
@@ -66,17 +64,21 @@ class Pixel:
         Pycroscopy requires different orientation, so this corrects for this effect.
     fit_form : str, optional
         Functional form used when fitting. 
+        
         One of 
             product: product of two exponentials (default)
             sum: sum of two exponentials
             exp: single expential decay
             ringdown: single exponential decay of amplitude, not frequency, scaled to return Q
+    
     method : str, optional
         Method for generating instantaneous frequency, amplitude, and phase response
-        One of 
+        
+        One of
             hilbert: Hilbert transform method (default)
             wavelet: Morlet CWT approach
             stft: short time Fourier transform (sliding FFT)
+    
     filter_amplitude : bool, optional
         The Hilbert Transform amplitude can sometimes have drive frequency artifact.
     filter_frequency : bool, optional
@@ -110,12 +112,6 @@ class Pixel:
     analyze()
         Analyzes signals and returns tfp, shift and inst_freq.
 
-    See Also
-    --------
-    line: Line processing for FF-trEFM data.
-    simulate: Simulation for synthetic FF-trEFM data.
-    scipy.signal.get_window: Windows for signal processing.
-
     Notes
     -----
     Frequency shift from wavelet analysis is not in Hertz. It should be used
@@ -127,10 +123,10 @@ class Pixel:
 
     References
     ----------
-    .. [1] Giridharagopal R, Rayermann GE, Shao G, et al. Submicrosecond time
+    .. \[1\] Giridharagopal R, Rayermann GE, Shao G, et al. Submicrosecond time
        resolution atomic force microscopy for probing nanoscale dynamics.
        Nano Lett. 2012;12(2):893-8.
-       [2] Karatay D, Harrison JA, et al. Fast time-resolved electrostatic
+       \[2\] Karatay D, Harrison JA, et al. Fast time-resolved electrostatic
        force microscopy: Achieving sub-cycle time resolution. Rev Sci Inst.
        2016;87(5):053702
 
@@ -151,10 +147,19 @@ class Pixel:
 
     """
 
-    def __init__(self, signal_array, params, can_params={},
-                 fit=True, pycroscopy=False,
-                 method='hilbert', fit_form='product', filter_amplitude=False,
-                 filter_frequency=False):
+    def __init__(self, 
+                 signal_array, 
+                 params={}, 
+                 can_params={},
+                 fit=True, 
+                 pycroscopy=False,
+                 method='hilbert', 
+                 fit_form='product', 
+                 filter_amplitude=False,
+                 filter_frequency=False,
+                 trigger=None,
+                 total_time=None,
+                 sampling_rate=None):
 
         # Create parameter attributes for optional parameters.
         # These defaults are overwritten by values in 'params'
@@ -178,6 +183,8 @@ class Pixel:
         self.recombination = False
         self.phase_fitting = False
         self.check_drive = True
+        self.window = 'blackman'
+        self.bandpass_filter = 1
 
         # Assign the fit parameter.
         self.fit = fit
@@ -194,40 +201,68 @@ class Pixel:
         self.Beta = 3114
         self.Q = 360
 
-        # Read parameter attributes from parameters dictionary.
-        for key, value in params.items():
-            setattr(self, key, value)
-
-        for key, value in can_params.items():
-            setattr(self, key, float(value))
-
-        if self.filter_frequency:
-            self.bandpass_filter = 0  # turns off FIR
-
-        # Assign values from inputs.
+        # Set up the array
         self.signal_array = signal_array
         self.signal_orig = None  # used in amplitude calc to undo any Windowing beforehand
-        if pycroscopy:
-            self.signal_array = signal_array.T
-        self.tidx = int(self.trigger * self.sampling_rate)
 
-        # Set dimensions correctly
-        # Three cases: 1) 2D (has many averages) 2) 1D (but set as 1xN) and 3) True 1D
         if len(signal_array.shape) == 2 and 1 not in signal_array.shape:
 
             self.n_points, self.n_signals = self.signal_array.shape
-            self._n_points_orig = self.signal_array.shape[0]
 
         else:
 
             self.n_signals = 1
             self.signal_array = self.signal_array.flatten()
             self.n_points = self.signal_array.shape[0]
-            self._n_points_orig = self.signal_array.shape[0]
+        
+        self._n_points_orig = self.signal_array.shape[0]
+        
+        if pycroscopy:
+            self.signal_array = signal_array.T
+        
+        setattr(self, 'trigger', trigger)
+        setattr(self, 'total_time', total_time)
+        setattr(self, 'sampling_rate', sampling_rate)
+        
+        # Read parameter attributes from parameters dictionary.
+        for key, value in params.items():
+            if not hasattr(self, key) or getattr(self, key) == None:
+                setattr(self, key, value)
 
-        # Keep the original values for restoring the signal properties.
+        for key, value in can_params.items():
+            setattr(self, key, float(value))
+
+        # Check for missing required parameters
+        if self.trigger == None: # trigger can be 0
+            raise KeyError('Trigger must be supplied')
+            
+        if not self.total_time:
+            if not self.sampling_rate:
+                raise KeyError('total_time or sampling_rate must be supplied')
+            else:
+                self.total_time = self.sampling_rate * self.n_points
+        elif not self.sampling_rate:
+            self.sampling_rate = int(self.n_points / self.total_time)
+        elif self.total_time != self.n_points / self.sampling_rate  :
+            print(self.n_points / self.sampling_rate )
+            print(self.total_time)
+            raise ValueError('total_time and sampling_rate mismatch')
+        
+        if not hasattr(self, 'roi'):
+            self.roi = 0.3 * (self.total_time - self.trigger)
+            warnings.warn('ROI defaulting to 30% post-trigger')
+        
+        self.tidx = int(self.trigger * self.sampling_rate)    
         self._tidx_orig = self.tidx
         self.tidx_orig = self.tidx
+        
+        if not hasattr(self, 'drive_freq'):
+            self.average()
+            self.set_drive()
+        
+        # Processing parameters    
+        if self.filter_frequency:
+            self.bandpass_filter = 0  # turns off FIR
 
         # Initialize attributes that are going to be assigned later.
         self.signal = None
@@ -237,8 +272,6 @@ class Pixel:
         self.shift = None
         self.cwt_matrix = None
 
-        self.verbose = False  # for console feedback
-
         # For accidental passing ancillary datasets from Pycroscopy, this will fail
         # when pickling
         if hasattr(self, 'Position_Indices'):
@@ -292,6 +325,20 @@ class Pixel:
 
         return
 
+    def set_drive(self):
+        """Calculates drive frequency of averaged signals"""
+        n_fft = 2 ** int(np.log2(self.tidx))  # For FFT, power of 2.
+        dfreq = self.sampling_rate / n_fft  # Frequency separation.
+
+        # Calculate drive frequency from maximum power of the FFT spectrum.
+        signal = self.signal[:n_fft]
+        fft_amplitude = np.abs(np.fft.rfft(signal))
+        drive_freq = fft_amplitude.argmax() * dfreq
+        
+        self.drive_freq = drive_freq
+        
+        return
+
     def check_drive_freq(self):
         """Calculates drive frequency of averaged signals, and check against
            the given drive frequency."""
@@ -326,7 +373,7 @@ class Pixel:
         rate = self.sampling_rate
         lpf = self.drive_freq * 0.1
         self.signal, _, _ = dwavelet.dwt_denoise(self.signal, lpf, rate / 2, rate)
-        
+
         return
 
     def fir_filter(self):
@@ -425,6 +472,8 @@ class Pixel:
         Filters the instantaneous frequency to remove noise
         Defaults to DC and then every multiple harmonic up to sampling
         
+        Parameters
+        ----------
         width : int, optional
             Size of the boxcar around the various peaks
         '''
@@ -485,8 +534,6 @@ class Pixel:
 
         phase = self.phase  # + np.pi/2 #offsets the phase to be pi/2 at resonance
 
-        # check for incorrect values (some off by 1e9 in our code)
-
         A = self.k / self.Q * self.amplitude ** 2 * (self.inst_freq + self.drive_freq)
         B = self.Q * self.DriveAmplitude * np.sin(phase) / self.amplitude
         C = self.inst_freq / self.drive_freq
@@ -549,32 +596,41 @@ class Pixel:
         cut -= self.inst_freq[self.tidx]
         self.cut = cut
         t = np.arange(cut.shape[0]) / self.sampling_rate
+        
+        try:
+            
+            if not self.fit:
+    
+                tfp_calc.find_minimum(self, cut)
+    
+            elif self.fit_form == 'sum':
+    
+                tfp_calc.fit_freq_sum(self, cut, t)
+    
+            elif self.fit_form == 'exp':
+    
+                tfp_calc.fit_freq_exp(self, cut, t)
+    
+            elif self.fit_form == 'ringdown':
+    
+                cut = self.amplitude[self.tidx:(self.tidx + ridx)]
+                tfp_calc.fit_ringdown(self, cut, t)
+    
+            elif self.fit_form == 'product':
+    
+                tfp_calc.fit_freq_product(self, cut, t)
+    
+            elif self.fit_form == 'phase':
+    
+                cut = -1 * (self.phase[self.tidx:(self.tidx + ridx)] - self.phase[self.tidx])
+                tfp_calc.fit_phase(self, cut, t)
 
-        if not self.fit:
-
-            tfp_calc.find_minimum(self, cut)
-
-        elif self.fit_form == 'sum':
-
-            tfp_calc.fit_freq_sum(self, cut, t)
-
-        elif self.fit_form == 'exp':
-
-            tfp_calc.fit_freq_exp(self, cut, t)
-
-        elif self.fit_form == 'ringdown':
-
-            cut = self.amplitude[self.tidx:(self.tidx + ridx)]
-            tfp_calc.fit_ringdown(self, ridx, cut, t)
-
-        elif self.fit_form == 'product':
-
-            tfp_calc.fit_freq_product(self, cut, t)
-
-        elif self.fit_form == 'phase':
-
-            cut = -1 * (self.phase[self.tidx:(self.tidx + ridx)] - self.phase[self.tidx])
-            tfp_calc.fit_phase(self, cut, t)
+        except:
+            
+            self.tfp = np.nan
+            self.shift = np.nan
+            self.best_fit = np.zeros(cut.shape[0])
+            print('error with fitting')
 
         return
 
@@ -622,6 +678,8 @@ class Pixel:
         
         wavelet specified in self.wavelet. See PyWavelets CWT documentation
         
+        Parameters
+        ----------
         Optimize : bool, optionals
             Currently placeholder for iteratively determining wavelet scales
         '''
@@ -635,6 +693,7 @@ class Pixel:
 
         dt = 1 / self.sampling_rate
         sc = pywt.scale2frequency(self.wavelet, self.scales) / dt
+        
         if self.verbose:
             print('Wavelet scale from', np.min(sc), 'to', np.max(sc))
 
@@ -693,10 +752,12 @@ class Pixel:
 
         return
 
-    def calculate_stft(self, time_res=20e-6, fit=False, nfft=200):
+    def calculate_stft(self, time_res=20e-6, nfft=200):
         '''
         Sliding FFT approach
         
+        Parameters
+        ----------
         time_res : float, optional
             What timescale to evaluate each FFT over
                   
@@ -725,26 +786,13 @@ class Pixel:
                                                    mode='magnitude')
 
         # Parabolic ridge finder
-        if not fit:
-            inst_freq, amplitude, _ = parab.ridge_finder(spectrogram, freq)
-
-        # slow serial curve fitting
-        else:
-
-            for c in range(spectrogram.shape[1]):
-
-                SIG = spectrogram[:, c]
-                if fit:
-                    pk = np.argmax(np.abs(SIG))
-                    popt = np.polyfit(freq[pk - 2:pk + 2], np.abs(SIG[pk - 2:pk + 2]), 2)
-                    inst_freq[c] = -0.5 * popt[1] / popt[0]
-                    amplitude[c] = np.abs(SIG)[pk]
+        inst_freq, amplitude, _ = parab.ridge_finder(spectrogram, freq)
 
         # Correctly pad the signals
         _pts = self.n_points - len(inst_freq)
-        _pre = int(np.floor(_pts/2))
-        _post = int(np.ceil(_pts/2)) 
-        
+        _pre = int(np.floor(_pts / 2))
+        _post = int(np.ceil(_pts / 2))
+
         inst_freq = np.pad(inst_freq, (_pre, _post))
         amplitude = np.pad(amplitude, (_pre, _post))
 
@@ -773,6 +821,8 @@ class Pixel:
         """ 
         Quick visualization of best_fit and cut.
         
+        Parameters
+        ----------
         newplot : bool, optional
             generates a new plot (True) or plots on existing plot figure (False)
         
@@ -814,6 +864,8 @@ class Pixel:
         """
         Generates the instantaneous frequency
         
+        Parameters
+        ----------
         timing : bool, optional
             prints the time to execute (for debugging)
 
@@ -827,7 +879,7 @@ class Pixel:
             t1 = time.time()
 
         # Remove DC component, first.
-        # self.remove_dc()
+        self.remove_dc()
 
         # Phase-lock signals.
         # self.phase_lock()
@@ -840,7 +892,6 @@ class Pixel:
 
         # Check the drive frequency.
         if self.check_drive:
-            
             self.check_drive_freq()
 
         # DWT Denoise
@@ -877,7 +928,6 @@ class Pixel:
 
             # Filter out oscillatory noise from amplitude
             if self.filter_amplitude:
-                
                 self.amplitude_filter()
 
         else:
@@ -888,7 +938,6 @@ class Pixel:
 
         # Filter out oscillatory noise from instantaneous frequency
         if self.filter_frequency:
-            
             self.frequency_filter()
 
         return self.inst_freq, self.amplitude, self.phase
@@ -911,7 +960,6 @@ class Pixel:
 
         # If it's a recombination image invert it to find minimum.
         if self.recombination:
-            
             self.inst_freq = self.inst_freq * -1
 
         # Find where the minimum is.
@@ -919,12 +967,10 @@ class Pixel:
 
         # Restore the length due to FIR filter being causal
         if self.method == 'hilbert':
-            
             self.restore_signal()
 
         # If it's a recombination image invert it to find minimum.
         if self.recombination:
-            
             self.inst_freq = self.inst_freq * -1
             self.best_fit = self.best_fit * -1
             self.cut = self.cut * -1
