@@ -3,7 +3,6 @@ from . import load
 from . import mechanical_drive
 from . import electric_drive
 
-__all__ = ['simulate', 
-		   'cantilever', 
+__all__ = ['cantilever', 
 		   'mechanical_drive',
-		   'electric_drve']
\ No newline at end of file
+		   'electric_drive']
\ No newline at end of file
